chrome.commands.onCommand.addListener((command) => {
    if (command === "open_template_menu") {
        storeActiveElementAndOpenPopup();
    }
});

// アクティブな要素を取得し、ストレージに保存してからポップアップを開く
function storeActiveElementAndOpenPopup() {
    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
        if (!tabs || tabs.length === 0) {
            console.log("アクティブなタブが見つかりませんでした");
            return;
        }

        chrome.scripting.executeScript({
            target: { tabId: tabs[0].id },
            function: () => {
                function findActiveElement() {
                    let el = document.activeElement;
                    console.log("現在の activeElement:", el);

                    function searchIframes() {
                        const iframes = document.querySelectorAll("iframe");
                        console.log("ページ内の iframe 数:", iframes.length);
                        for (let iframe of iframes) {
                            try {
                                const doc = iframe.contentDocument || iframe.contentWindow.document;
                                if (doc) {
                                    const iframeActive = doc.activeElement;
                                    console.log("iframe 内の activeElement:", iframeActive);
                                    if (iframeActive && !["BODY", "HTML"].includes(iframeActive.tagName)) {
                                        return iframeActive;
                                    }
                                }
                            } catch (e) {
                                console.warn("iframe アクセス不可", e);
                            }
                        }
                        return null;
                    }

                    if (!el || el.tagName === "BODY") {
                        console.log("activeElement が BODY なので iframe を検索");
                        el = searchIframes();
                    }

                    if (!el) {
                        console.log("ページ内の最初の textarea または input を取得");
                        el = document.querySelector("textarea, input, [contenteditable='true']");
                    }

                    // `contenteditable="true"` も対象にする
                    if (el && (el.tagName === "INPUT" || el.tagName === "TEXTAREA" || el.isContentEditable)) {
                        console.log("有効なアクティブ要素:", el);
                        return {
                            tagName: el.tagName,
                            id: el.id || null,
                            name: el.name || null,
                            isContentEditable: el.isContentEditable,
                            index: Array.from(document.querySelectorAll(el.tagName)).indexOf(el)
                        };
                    }

                    console.log("適切な入力フィールドが見つかりませんでした");
                    return null;
                }

                return findActiveElement();
            }
        }, (results) => {
            if (!results || !results[0] || !results[0].result) {
                console.log("アクティブな要素が取得できませんでした（深掘り検索後も失敗）");
                return;
            }

            console.log("取得したアクティブ要素:", results[0].result);

            chrome.storage.local.set({ activeElement: results[0].result }, () => {
                console.log("アクティブな要素をストレージに保存:", results[0].result);
                verifyStorage(results[0].result);
            });
        });
    });
}

// ストレージに保存されたデータを確認し、ポップアップを開く
function verifyStorage(expectedData, attempts = 10) {
    chrome.storage.local.get(["activeElement"], (checkResult) => {
        console.log("ストレージから取得:", checkResult.activeElement);
        console.log("期待されるデータ:", expectedData);

        if (JSON.stringify(checkResult.activeElement) === JSON.stringify(expectedData)) {
            console.log("データ一致 ポップアップを開く");
            chrome.action.openPopup();
        } else if (attempts > 0) {
            console.log("データが一致しない 再試行", attempts);
            setTimeout(() => verifyStorage(expectedData, attempts - 1), 50);
        } else {
            console.log("ストレージのデータが一致しなかった");
        }
    });
}

/** 追加: ストレージに保存されるデータの確認ログを強化 */
function debugStorage() {
    chrome.storage.local.get(["activeElement"], (result) => {
        console.log("デバッグ用 - ストレージのアクティブ要素:", result.activeElement);
    });
}
